/**
 * Copyright (C) 2022 Xilinx, Inc
 * Copyright (C) 2022 Advanced Micro Devices, Inc. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may
 * not use this file except in compliance with the License. A copy of the
 * License is located at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

// ------ I N C L U D E   F I L E S -------------------------------------------
// Local - Include Files
#include "ReportThermal.h"
#include "MAUtilities.h"

// 3rd Party Library - Include Files
#include <string>
#include <boost/format.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/property_tree/json_parser.hpp>

#define BYTES_TO_MEGABYTES 0x100000ll

void ReportThermal::getPropertyTree(const std::shared_ptr<MA35Device> dev, boost::property_tree::ptree& _pt) const {
  boost::property_tree::ptree pt;
  XBUtilities::get_thermal_info(dev, pt);

  // There can only be 1 root node
  _pt.add_child("thermal", pt);
}

void printThermal(const boost::property_tree::ptree& pt, std::ostream& _output, bool max) {
  std::string name     = "Device Temp";
  int32_t     max_temp = -1;
  for (const auto& kd : pt) {
    const auto& t1   = kd.second;
    auto        id   = t1.get<std::string>("id", "");
    auto        temp = t1.get<int32_t>("temp_C", -1);
    if (id.empty())
      id = kd.first;

    if (!max) {
      _output << "  id: " << id << " [" << t1.get<std::string>("temp_C", "?") << " C]" << std::endl;
    } else {
      if (temp > max_temp) {
        max_temp = temp;
        //name     = id;
      }
    }
  }
  if (max) {
    _output << "  id: " << name << " [" << max_temp << " C]" << std::endl;
  }
}
void ReportThermal::writeReport(const boost::property_tree::ptree& _pt, std::ostream& _output) const {
  boost::property_tree::ptree empty_ptree;

  _output << "MA35 Thermal Info:\n";
  if (_pt.get<std::string>("thermal.status") != "ok") {
    _output << "ERROR " << _pt.get<std::string>("thermal.status") << std::endl;
    return;
  }

  try {
    const boost::property_tree::ptree& temp1 = _pt.get_child("thermal.temp.chip", empty_ptree);
    const boost::property_tree::ptree& temp2 = _pt.get_child("thermal.temp.board", empty_ptree);
    _output << "Device Temperature:\n";
    printThermal(temp1, _output, true);
    _output << "Board Temperature:\n";
    printThermal(temp2, _output, false);

  } catch (const boost::property_tree::ptree_error& ex) {
    _output << "ERROR: Unexpected error" << std::endl;
  }
  _output << std::endl;
}
